<?php

namespace ElfsightVimeoGalleryApi;


if (!defined('ABSPATH')) exit;

require_once __DIR__ . '/vendor/autoload.php';

class Api extends Core\Api {
    private $routes = array(
        '' => 'requestController',
        'auth' => 'authController'
    );

    const API_BASE_URL = 'https://api.vimeo.com/';
    const AUTH_BASE_URL = 'https://storage.elfsight.com/auth/vimeo/';

    static $ERROR_USER_UPDATE;
    static $ERROR_USER_NOT_FOUND;
    static $ERROR_USER_NOT_AUTH;

    public function __construct($config) {
        self::$ERROR_USER_UPDATE = __('Can\'t update user data');
        self::$ERROR_USER_NOT_FOUND = __('User not found');
        self::$ERROR_USER_NOT_AUTH = __('User not authorized');

        parent::__construct($config, $this->routes);
    }

    public function authController() {
        $identity = $this->input('user_id');
        $token = $this->input('token', false, false);

        if (empty($token)) {
            $auth_url = self::AUTH_BASE_URL;
            $redirect_url = (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? 'https' : 'http') . "://$_SERVER[HTTP_HOST]$_SERVER[REQUEST_URI]";

            $auth_url = $this->Helper->addQueryParam($auth_url, 'identity', $identity);
            $auth_url = $this->Helper->addQueryParam($auth_url, 'redirect_url', urlencode($redirect_url));

            header("Location: $auth_url");
            exit();
        } else {
            $json_data = json_encode(array(
                'token' => $this->input('token', false, true)
            ));

            $provider = $this->input('provider', null, true);

            if ($this->User->set($identity, $json_data)) {
                $response = "<script>window.opener.postMessage({'action': 'token', 'callback': 'authorize', 'status': 1, 'identity': '$identity', 'provider': '$provider', 'data': $json_data}, '*'); window.close();</script>";

                return $this->response($response, array('plain' => true));
            } else {
                $this->error(400, self::$ERROR_INVALID_AUTH, self::$ERROR_USER_UPDATE);
            }
        }
    }

    public function requestController() {
        $q = $this->input('q');

        $cache_key = $this->Cache->keyFromQuery($q, array('fields'));
        $data = $this->Cache->get($cache_key);

        if (empty($data)) {
            $request_url = $this->Url->setRequestUrl($q, self::API_BASE_URL);

            $user_id = $this->input('user_id');
            $user = $this->User->get($user_id);

            if (empty($user)) {
                $this->error(400, self::$ERROR_INVALID_AUTH, self::$ERROR_USER_NOT_FOUND);
            }

            $user_data = json_decode($user['data'], true);

            if (empty($user_data['token'])) {
                $this->error(400, self::$ERROR_INVALID_AUTH, self::$ERROR_USER_NOT_AUTH);
            }

            $response = $this->request('GET', $request_url, array(
                'headers' => array(
                    'Authorization' => "Bearer $user_data[token]"
                )
            ));

            if ($this->checkResponse($response, false)) {
                $data = $response['body'];
                $data_arr = json_decode($response['body'], true);

                if (!empty($data_arr['error'])) {
                    return $this->error($data_arr['error_code'], $data_arr['error']);
                }

                if ((int) $response['http_code'] === 200) {
                    $this->Cache->set($cache_key, $data);
                }
            } else {
                return $this->error();
            }
        }

        return $this->response($data);
    }
}
